"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tracker_proxy_1 = require("tracker-proxy");
const child_process_1 = require("child_process");
class TorrentProxyServer {
    constructor(config) {
        this.peerflix = null;
        this.config = config;
        this.trackerManager = new tracker_proxy_1.Manager();
        this.trackerManager.include(new tracker_proxy_1.adapters.TPB());
    }
    search(query) {
        return new Promise((resolve, reject) => {
            this.trackerManager.search({ query: query });
            this.trackerManager.on(this.trackerManager.EVENT_SEARCH_RESPONSE, (tracker, torrents) => {
                const collection = torrents.map((torrent) => {
                    return {
                        title: torrent.title,
                        magnet: torrent.magnet
                    };
                });
                resolve(collection);
            });
            this.trackerManager.on(this.trackerManager.EVENT_SEARCH_ERROR, (tracker, err) => {
                reject(err);
            });
        });
    }
    load(magnet) {
        return new Promise((resolve, reject) => {
            return this.cancel()
                .then(() => {
                const localIP = this.config.ip;
                const port = this.config.port;
                this.peerflix = child_process_1.spawn('peerflix', ['--hostname', localIP, '--port', port, magnet]);
                this.peerflix.on('error', (err) => reject(err));
                let resolved = false;
                this.peerflix.stdout.on('data', () => {
                    if (resolved) {
                        return;
                    }
                    resolved = true;
                    resolve('http://' + localIP + ':' + port);
                });
                this.peerflix.stderr.on('data', (err) => {
                    if (err) {
                        reject(err);
                    }
                });
            });
        });
    }
    cancel() {
        if (!this.peerflix) {
            return Promise.resolve();
        }
        this.peerflix.kill('SIGTERM');
        this.peerflix = null;
        return Promise.resolve();
    }
}
exports.default = TorrentProxyServer;
//# sourceMappingURL=index.js.map